﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Nautilus;

namespace cs
{
    class Program
    {
        static void Main(string[] args)
        {
            // Load a driver license text. Don't forget to put testdl.txt next to exe file.
            var scannedText = File.ReadAllText(@"testdl.txt");
            // Parse the text and create an instance of DriverLicense.
            // All fields in DriverLicense are strings. 
            // When the parser is not able to parse the text it returns null. Parser doesn't throw exceptions.
            var license = DriverLicense.ParseText(scannedText);

            if (license != null)
            {
                //This functions demonstrates very simple example
                Console.WriteLine("------Basic Usage--------");
                BasicUsage(license);

                // If you need to enumarate all field you can use DataElementAttribute and the reflection
                Console.WriteLine("------Print All Fields--------");
                PrintAllFields(license);
            }

            // However, if you need typed fields or you need to know what was really found in the license you should use DriverLicenseEx
            // It does the same job but the filed types can be string, DataTime?, Char? or bool?
            // If field was not present in the license the field is null.
            var licenseEx = DriverLicenseEx.ParseText(scannedText);
            if (licenseEx != null)
            {
                Console.WriteLine("------Print All Found Fields--------");
                PrintAllFoundFields(licenseEx);
            }
        }
        private static void BasicUsage(DriverLicense license)
        {
            // Basic Usage
            Console.WriteLine(license.FirstName);
            Console.WriteLine(license.LastName);
            Console.WriteLine();
        }

        private static void PrintAllFields(DriverLicense license)
        {
            //Enumerate all fields
            var properties = license.GetType().GetProperties();
            var foundFields = (from info in properties
                               let element = (DataElementAttribute[])info.GetCustomAttributes(typeof(DataElementAttribute), false)
                               where element.Length > 0
                               select info).ToList();

            foundFields.Sort((x, y) => string.Compare(x.Name, y.Name, StringComparison.Ordinal));

            for (var i = 0; i < foundFields.Count; i++)
            {
                var fieldName = foundFields[i].Name;
                var fieldValue = foundFields[i].GetValue(license, null);
                Console.ForegroundColor = ConsoleColor.White;
                Console.Write("{0,-3}{1,-30}", i + 1, fieldName);
                Console.ForegroundColor = ConsoleColor.Yellow;
                if (fieldName.EndsWith("Date") && !string.IsNullOrWhiteSpace(fieldValue.ToString()))
                {
                    DateTime dt;
                    var res = DateTime.TryParseExact(fieldValue.ToString(), "MMddyyyy", CultureInfo.InvariantCulture, DateTimeStyles.None, out dt);
                    Console.WriteLine("{0}", res ? dt.ToString("MM/dd/yyyy") : fieldValue);
                }
                else
                {
                    Console.WriteLine("{0}", fieldValue);
                }
            }
        }

        private static void PrintAllFoundFields(DriverLicenseEx license)
        {
            //Enumerate all  fields
            var properties = license.GetType().GetProperties();
            var foundFields = (from info in properties
                               let element = (DataElementAttribute[])info.GetCustomAttributes(typeof(DataElementAttribute), false)
                               where element.Length > 0
                               select info).ToList();

            foundFields.Sort((x, y) => string.Compare(x.Name, y.Name, StringComparison.Ordinal));

            for (var i = 0; i < foundFields.Count; i++)
            {
                var fieldName = foundFields[i].Name;
                var fieldValue = foundFields[i].GetValue(license, null);
                Console.ForegroundColor = ConsoleColor.White;
                Console.Write("{0,-3}{1,-30}", i + 1, fieldName);
                Console.ForegroundColor = ConsoleColor.Yellow;
                Console.WriteLine("{0}", fieldValue?.ToString() ?? "N/A");
            }
        }

    }
}
